use base::*;

#[derive(Debug, PartialEq, Clone, Copy)]
enum Move {
    Rock,
    Paper,
    Scissors
}

enum GameResult {
    Win,
    Draw,
    Lose
}

impl GameResult {
    pub fn from_str(c: &str) -> GameResult {
        return match c {
            "X" => {GameResult::Lose}
            "Y" => {GameResult::Draw}
            "Z" | _ => {GameResult::Win}
        }
    }
}

impl Move {
    pub fn from_str(c: &str) -> Move {
        return match c {
            "A" | "X" => {
                Move::Rock
            }
            "B" | "Y" => {
                Move::Paper
            }
            "C" | "Z" | _ => {
                Move::Scissors
            }
        }
    }

    pub fn score(self) -> u32 {
        return match self {
            Move::Rock => {1}
            Move::Paper => {2}
            Move::Scissors => {3}
        }
    }

    pub fn result(self, other: Move) -> GameResult {
        if self == other {
            return GameResult::Draw;
        }
        match self {
            Move::Rock => {
                if other == Move::Scissors {
                    return GameResult::Win
                }
            }
            Move::Paper => {
                if other == Move::Rock {
                    return GameResult::Win
                }
            }
            Move::Scissors => {
                if other == Move::Paper {
                    return GameResult::Win
                }
            }
        }

        GameResult::Lose
    }
}

struct GameMove(Move, Move);

impl GameMove {
    fn calc_score(&self) -> u32 {
        let result = self.1.result(self.0);
        let score = self.1.score();
        return match result {
            GameResult::Win => {
                score + 6
            }
            GameResult::Draw => {
                score  + 3
            }
            GameResult::Lose => {
                score
            }
        }
    }

    fn from_opponent_move(opponent_move: Move, result: GameResult) -> GameMove {
        return match result {
            GameResult::Draw => {
                GameMove(opponent_move, opponent_move)
            }
            GameResult::Win => {
                let our_move = match opponent_move {
                    Move::Rock => {Move::Paper}
                    Move::Paper => {Move::Scissors}
                    Move::Scissors => {Move::Rock}
                };
                GameMove(opponent_move, our_move)
            }
            GameResult::Lose => {
                let our_move = match opponent_move {
                    Move::Rock => {Move::Scissors}
                    Move::Paper => {Move::Rock}
                    Move::Scissors => {Move::Paper}
                };
                GameMove(opponent_move, our_move)
            }
        }
    }
}

fn main() {
    let lines = read_file("day2.txt");

    let sum = lines.iter().map(|line| {
        let symbols_per_move = line.split(" ");
        let mut symbols_iter = symbols_per_move.map(|symbol| Move::from_str(symbol)).take(2);

        GameMove(symbols_iter.next().unwrap(), symbols_iter.next().unwrap())
    }).map(|game_move|game_move.calc_score()).sum::<u32>();

    println!("Task 1: Got sum: {sum}");

    let sum = lines.iter().map(|line| {
        let mut symbols_per_move = line.split(" ");

        GameMove::from_opponent_move(Move::from_str(symbols_per_move.next().unwrap()), GameResult::from_str(symbols_per_move.next().unwrap()))
    }).map(|game_move|game_move.calc_score()).sum::<u32>();


    println!("Task 2: Got sum: {sum}");

}